@@ -45,6 +45,7 @@ import java.lang.ref.SoftReference;
 import java.io.File;
 import java.net.Authenticator;
 import java.net.PasswordAuthentication;
+import java.util.Map;
 
 /**
  *  This class generates an image or images from user inputs.
@@ -91,6 +92,7 @@ import java.net.PasswordAuthentication;
  *              <li> Tan: The tangent plane on gnomonic projection.
  *              <li> Sin: The sine or orthographic projection.
  *              <li> Ait: The Hammer-Aitoff projection.
+ *              <li> Mol: MillWeide projection.
  *              <li> Car: The cartesian or plate-caree projection.
  *              <li> Zea: The Zenithal equal area projection
  *              <li> Csc: The COBE sperical cube projection.
@@ -569,7 +571,7 @@ public class Imager {
 	Settings.put("output", path);
     }
     
-    protected Survey loadSurvey(String surveyID) throws Exception {
+    public Survey loadSurvey(String surveyID) throws Exception {
 	
 	Survey surv;
 	//Survey surv = finder.find(surveyID);
@@ -678,7 +680,7 @@ public class Imager {
 	return new Image(data, wcs, nx, ny, nz);
     }
     
-    protected Image[] loadCandidates(Position pos) throws Exception {
+    protected Image[] loadCandidates(Position pos, Survey surv) throws Exception {
         // Get max scale using scale and image size
 	double maxSize = Math.max(nx, ny)*wcs.getScale()*180/Math.PI;
 	if (maxSize <= 0) {
@@ -747,17 +749,33 @@ public class Imager {
     }
     
     protected void doProcess(String type) throws Exception {
+        doProcess(type, null);
+    }
+    
+    protected void doProcess(String type, Boolean updates) throws Exception {
 	
 	String[] procNames = Settings.getArray(type);
 	for (int i=0; i<procNames.length; i += 1) {
-	    dynoProcess(procNames[i]);
+	    dynoProcess(procNames[i], updates);
 	}
     }
     
-    public void dynoProcess(String name) throws Exception {
+    /** Run a processor.
+     * 
+     * @param name     The class name of the processor
+     * @param updates  Indicates whether the processor should be run.
+     *                 If null, then any processor should run.
+     *                 Otherwise run the processor only if the values of updates
+     *                 is the same as the value of the updatesOutput method of the
+     *                 processor.
+     */
+    public void dynoProcess(String name, Boolean updates) throws Exception {
 	Processor proc = (Processor) Class.forName(name).newInstance();
-	proc.process(cand, output, match, samp, dsamp);
-	processes.add(proc);
+        if (updates == null || (proc.updatesOutput() == updates)) {
+	    proc.process(cand, output, match, samp, dsamp);
+	    processes.add(proc);
+        } else {
+        }
     }
     
     /** Clear out state that may confuse subsequent processing */
@@ -780,12 +798,64 @@ public class Imager {
 	    return null;
 	}
     }
+    
+    Image[] processComponents(String surveyID, String[] components) throws Exception {
+        
+        // Pop out the settings for the current (compound) survey
+        // so that they don't interfere with the settings of the
+        // components.
+        HashMap<String,String> originalSettings = Settings.pop();
+        
+        String[] elements = surveyID.split(" ");
+        String base = "";
+        for (int i = 0; i < elements.length - 1; i += 1) {
+            base += elements[i] + " ";
+        }
+        Image[] comps = new Image[components.length];
+        Image oldOutput = output;
+        Survey oldSurv = surv;
+        for (int i = 0; i < components.length; i += 1) {
+            
+            // Save this set of settings so we can get back to this
+            // for each component.
+            Settings.save();
+            String xid = base + components[i];
+            System.err.println("  Processing survey component: "+components[i]);
+            surv = loadSurvey(xid);
+            output = loadImage();
+            Settings.put("_currentSurvey", xid);
+            output = doProcessSurvey(xid, surv);
+            postprocessSurvey(true);  // Only run the updating methods.
+            comps[i] = output;
+            Settings.restore();
+        }
+        output = oldOutput;
+        surv   = oldSurv;
+        Settings.push(originalSettings);
+        return comps;
+    }
 	    
     /** Process a particular survey. */
     public void processSurvey(String surveyID) throws Exception {
         Settings.put("_currentSurvey", surveyID);
-	output = loadAndProcessSurvey(surveyID);
-	postprocessSurvey();
+        surv   = loadSurvey(surveyID);
+        String[] components = Settings.getArray("SurveyComponents");
+        if (components != null && components.length > 0) {
+            wcs    = loadWCS();
+            output = loadImage();
+            // Create the component elements and make them the 'candidate' images 
+            // for the compound survey
+            cand = processComponents(surveyID, components);
+            String combiner = Settings.get("SurveyCombiner");            
+            // Combine the components.
+            dynoProcess(combiner, null);
+            // Now run non-updating posprocessors
+            postprocessSurvey(false);
+            
+        } else {
+	    output = doProcessSurvey(surveyID, surv);
+	    postprocessSurvey(null);
+        }
 	if (match != null && output != null  && !Settings.has("nofits")) {
 	   createFitsFile();
 	}
@@ -797,10 +867,9 @@ public class Imager {
 	return sum;
     }
     
-    public Image loadAndProcessSurvey(String surveyID) throws Exception {
+    public Image doProcessSurvey(String surveyID, Survey surv) throws Exception {
 	
 	processes = new ArrayList<Processor>();
-	surv = loadSurvey(surveyID);
 	if (surv == null) {
 	    return null;
 	}
@@ -824,7 +893,7 @@ public class Imager {
 	}
 	parseEbins();
 	Image prime  = loadImage();
-	cand         = loadCandidates(pos);
+	cand         = loadCandidates(pos, surv);
 	
 	// Allow candidate selection and mosaicking using
 	//  subset images.
@@ -851,6 +920,7 @@ public class Imager {
 	    if (Settings.get("Mosaicker") == null) {
 		Settings.put("Mosaicker", "skyview.process.Mosaicker");
 	    }
+
 	    Processor mos = (Processor) Class.forName(Settings.get("Mosaicker")).newInstance();
 	    processes.add(mos);
 	
@@ -863,11 +933,12 @@ public class Imager {
 	    }
 	
 	    if (match != null) {
-	        loadSamplers();
+	        loadSamplers();                
 	        doProcess("Preprocessor");
 		mos.process(cand, output, match, samp, dsamp);
 	    }
 	}
+        
 	
 	// Subsequent processing should take place using the full image,
 	// not the image subsets.
@@ -880,12 +951,11 @@ public class Imager {
 	return output;
     }
     
-    public void postprocessSurvey() throws Exception {
+    public void postprocessSurvey(Boolean allProc) throws Exception {
 
 	// Do we have any postprocesing? (Maybe even if the image failed)
-	// Use lastMatch rather than match, since match may have been consumed by the mosaicker.
 	doProcess("Deedger");
-	doProcess("Postprocessor");
+	doProcess("Postprocessor", allProc);
 	cleanCache();
         dsamp = null;
     }
